/**@file

// cpcDigitalParameter.ctl
This library contains the widget, faceplate, etc. functions of DigitalParameter.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{


/** Function called from snapshot utility of the treeDeviceOverview to get the time and value
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name
@param deviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
CPC_DigitalParameter_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
  float value;
  string sTime;

	dpGet(deviceName+".ProcessInput.StsReg01", value,
        deviceName+".ProcessInput.StsReg01:_online.._stime", sTime);

  dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(deviceName + ".ProcessInput.StsReg01"), value) + " " + dpGetUnit(deviceName + ".ProcessInput.StsReg01"); // formated value with unit
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}

/** pop-up menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param dsAccessOk input, the access control 
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
CPC_DigitalParameter_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_MASK_POSST_TEXT,
			 UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	cpcGenericObject_addUnicosActionToMenu(deviceName, dpType, dsMenuConfig, dsAccessOk, menuList);
	cpcGenericObject_addDefaultUnicosActionToMenu(deviceName, dpType, dsMenuConfig, dsAccessOk, menuList);
	cpcGenericObject_addTrendActionToMenu(deviceName, dpType, dsMenuConfig, dsAccessOk, menuList);
//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** handle the answer of the popup menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control 
@param menuAnswer input, selected menu value 
*/
CPC_DigitalParameter_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer)
{
  dyn_string exceptionInfo;

//begin_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
	cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_HandleMenu

  if (dynlen(exceptionInfo) > 0) {
      unSendMessage_toExpertException("Right Click",exceptionInfo);
  }
}

/**  widget register callback

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
CPC_DigitalParameter_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, systemName, frontEnd;
  int iAction, iRes;
  dyn_string exceptionInfo;
  bool bRemote;

  deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
  if (deviceName == "") deviceName = g_sDpName; // In case of disconnection
  systemName = unGenericDpFunctions_getSystemName(deviceName);
  frontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
  if (frontEnd != "") frontEnd = systemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+frontEnd;

  unDistributedControl_isRemote(bRemote, systemName);
  g_bSystemConnected = bRemote ? bSystemConnected : true;
  g_bUnSystemAlarmPlc = frontEnd != "" && dpExists(frontEnd);
  if (deviceName != "") {
    if (dpTypeName(deviceName) != UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME) g_bUnSystemAlarmPlc = false;
  }

  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  dyn_string lockDPEs;
  switch (iAction) {
    case UN_ACTION_DISCONNECT:
      CPC_DigitalParameter_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_DPCONNECT:
      g_sDpName = deviceName;
      if (g_bUnSystemAlarmPlc) {
        lockDPEs = cpcGenericAnimation_WidgetLockDPEs(deviceName);
        if (dynlen(lockDPEs) > 0) {
          iRes = dpConnect("cpcGenericAnimation_WidgetLockCallBack", lockDPEs);
          g_bCallbackConnected = (iRes >= 0);
        } else {
          cpcGenericAnimation_WidgetConnect(deviceName, frontEnd);
        }
      } else {
        CPC_DigitalParameter_WidgetDisconnection(g_sWidgetType);
      }
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      lockDPEs = cpcGenericAnimation_WidgetLockDPEs(deviceName);
      if (dynlen(lockDPEs) > 0) {
        if (g_bUnSystemAlarmPlc) {
          iRes = dpDisconnect("cpcGenericAnimation_WidgetLockCallBack", lockDPEs);
        }
        if (g_bDisconnectCB) {
          cpcGenericAnimation_WidgetDisconnect(deviceName, frontEnd);
        }
        g_bCallbackConnected = !(iRes >= 0);
        g_bDisconnectCB = false;
      } else {
        cpcGenericAnimation_WidgetDisconnect(deviceName, frontEnd);
      }
      CPC_DigitalParameter_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_NOTHING:
      break;
    default:
      break;
  }
}

/** Returns the list of DigitalParameter DPEs which should be connected on widget animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_DigitalParameter_WidgetDPEs(string deviceName, dyn_string &dpes)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names 
@param values input, the dpe values

*/
CPC_DigitalParameter_WidgetAnimation(dyn_string dpes, dyn_anytype values, string widgetType)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
  int iSystemIntegrityAlarmValue = values[1];
  bool bSystemIntegrityAlarmEnabled = values[2];
  bool bLocked = values[3];
  string sSelectedManager = values[4];
  int iMask = values[5];

  bit32 bit32StsReg01 = values[6];
  bool bStsReg01Invalid = values[7];

  string sbody1Color, sbody2Color, body1fill = "[outline]", body2fill = "[outline]";
	string selectColor, sWarningLetter, sWarningColor;
	bool bLockVisible;
  
  if (bStsReg01Invalid || cpcGenericObject_SystemInvalid(bSystemIntegrityAlarmEnabled, iSystemIntegrityAlarmValue)) {
		sbody1Color = "unDataNotValid";
		sbody2Color = "unDataNotValid";
  } else {
		sbody1Color = "cpcColor_Widget_Status";
		sbody2Color = "cpcColor_Widget_Request";
    if (getBit(bit32StsReg01, UN_STSREG01_POSST) == 1) {
      body1fill = "[solid]";
    }
    if (getBit(bit32StsReg01, CPC_STSREG01_MONRST) == 1) {
      body2fill = "[solid]";
    }
  }

  cpcGenericObject_WidgetValidnessAnimation(bSystemIntegrityAlarmEnabled, iSystemIntegrityAlarmValue,
                                           bStsReg01Invalid, 
                                           sWarningLetter, sWarningColor);
	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, bLockVisible);
	if(g_bSystemConnected) {
		setMultiValue("WarningText","text",sWarningLetter,
                 "WarningText","foreCol",sWarningColor,
                 "SelectArea", "foreCol", selectColor,
                 "LockBmp", "visible", bLockVisible,
                 "WidgetArea", "visible", true,
                 "Body1", "foreCol", sbody1Color, "Body1", "backCol", sbody1Color, "Body1", "fill", body1fill,
                 "Body2", "foreCol", sbody2Color, "Body2", "backCol", sbody2Color, "Body2", "fill", body2fill,
                 "eventState", "visible", iMask == 0);
	}
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}

/** Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_DigitalParameter_WidgetDisconnection(string sWidgetType)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
	setMultiValue("Body1","foreCol","unDataNoAccess",
                "Body1","backCol","unWidget_Background",
                "Body1","fill","[solid]",
                "Body2","foreCol","unDataNoAccess",
                "Body2","backCol","unWidget_Background",
                "Body2","fill","[solid]",
                "WarningText", "text", "", 
                "SelectArea", "foreCol", "", 
                "LockBmp", "visible", false,
                "WidgetArea", "visible", false,
                "eventState", "visible", false);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}

/** Return button configuration including access level
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
mapping CPC_DigitalParameter_ButtonConfig(string deviceName)
{
  mapping buttons;
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
  buttons[UN_FACEPLATE_BUTTON_SELECT] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[CPC_FACEPLATE_BUTTON_DIGITALPARAMETER_SET_VALUE] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[UN_FACEPLATE_MASKEVENT] = CPC_ACCESS_RIGHTS_EXPERT;
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
  return buttons;
}

/** Configure the list of dpes that needs for buttons animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the name of device
@param dpes input, the list of dpes to connect
*/
CPC_DigitalParameter_ButtonDPEs(string deviceName, dyn_string &dpes)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
}

/** Set the state of the contextual button of the device
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param dpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
CPC_DigitalParameter_ButtonSetState(string deviceName, string dpType, dyn_string dsUserAccess, dyn_string dsData)
{
  bool bSelected, buttonEnabled, bit1, bit2, bit3;
  string localManager;
  dyn_string exceptionInfo;

  bool bLocked = dsData[1];
  string sSelectedUIManager = dsData[2];
  
  localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
  bSelected = (localManager == "S"); // Selection state
  dyn_string dsButtons = cpcGenericAnimation_ButtonList(deviceName);

//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
  mixed stsReg01Value = dsData[3];
  bool bStsReg01Bad = dsData[4];
  for(int i = 1; i <= dynlen(dsButtons); i++) {
      buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access

      if (dsButtons[i] == UN_FACEPLATE_BUTTON_SELECT) {
        unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
      } else {
        cpcButton_setButtonState(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], buttonEnabled && bSelected);
      }

  }
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
}

/** Returns the list of DigitalParameter DPEs which should be connected on faceplate animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_DigitalParameter_FaceplateDPEs(string deviceName, dyn_string &dpes)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
  dynAppend(dpes, deviceName + ".ProcessOutput.DfltVal");
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
}

/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names 
@param values input, the dpe values

*/
CPC_DigitalParameter_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_POSST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "CPC_STSREG01_MONRST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "CPC_STSREG01_MOFFRST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "CPC_STSREG01_ARMRCPST", true, "cpcColor_Faceplate_Status");
  cpcGenericObject_CheckboxAnimate("DefaultValue", cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessOutput.DfltVal"));
  cpcFaceplate_animateMaskEvent(dpes, values);
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
}

//@}