/**@file

// cpcProcessControlObject.ctl
This library contains the widget, faceplate, etc. functions of ProcessControlObject.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{


/** Returns the list of ProcessControlObject DPE with alarm config that can be acknowledged
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name DP name
@param dpType input, device type
@param dsNeedAck output, the lsit of DPE
*/
CPC_ProcessControlObject_AcknowledgeAlarm(string deviceName, string dpType, dyn_string &dsNeedAck)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
  dsNeedAck = makeDynString("ProcessInput.FuStopISt", "ProcessInput.StartISt", "ProcessInput.TStopISt", "ProcessInput.AlSt", UN_ACKNOWLEDGE_PLC);
//end_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
}

/** Function called from snapshot utility of the treeDeviceOverview to get the time and value
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name
@param deviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
CPC_ProcessControlObject_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
  int value;
  string sTime, result;

  dpGet(deviceName+".ProcessInput.OpMoSt", value,
        deviceName+".ProcessInput.OpMoSt:_online.._stime", sTime);

  CPC_ProcessControlObject_getOptionModesLabel(deviceName, value, result);
  dsReturnData[1] = sTime;
  dsReturnData[2] = result;
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}

/** pop-up menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param dsAccessOk input, the access control 
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
CPC_ProcessControlObject_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
  dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, CPC_POPUPMENU_RECIPES_TEXT);

  cpcGenericObject_addUnicosActionToMenu(deviceName, dpType, dsMenuConfig, dsAccessOk, menuList);
  cpcGenericObject_addDefaultUnicosActionToMenu(deviceName, dpType, dsMenuConfig, dsAccessOk, menuList);
//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** handle the answer of the popup menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control 
@param menuAnswer input, selected menu value 
*/
CPC_ProcessControlObject_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer)
{
  dyn_string exceptionInfo;

//begin_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
  cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_HandleMenu

  if (dynlen(exceptionInfo) > 0) {
      unSendMessage_toExpertException("Right Click",exceptionInfo);
  }
}

/**  widget register callback

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
CPC_ProcessControlObject_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, systemName, frontEnd;
  int iAction, iRes;
  dyn_string exceptionInfo;
  bool bRemote;

  deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
  if (deviceName == "") deviceName = g_sDpName; // In case of disconnection
  systemName = unGenericDpFunctions_getSystemName(deviceName);
  frontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
  if (frontEnd != "") frontEnd = systemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+frontEnd;

  unDistributedControl_isRemote(bRemote, systemName);
  g_bSystemConnected = bRemote ? bSystemConnected : true;
  g_bUnSystemAlarmPlc = frontEnd != "" && dpExists(frontEnd);
  if (deviceName != "") {
    if (dpTypeName(deviceName) != UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME) g_bUnSystemAlarmPlc = false;
  }

  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  dyn_string lockDPEs;
  switch (iAction) {
    case UN_ACTION_DISCONNECT:
      CPC_ProcessControlObject_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_DPCONNECT:
      g_sDpName = deviceName;
      if (g_bUnSystemAlarmPlc) {
        lockDPEs = cpcGenericAnimation_WidgetLockDPEs(deviceName);
        if (dynlen(lockDPEs) > 0) {
          iRes = dpConnect("cpcGenericAnimation_WidgetLockCallBack", lockDPEs);
          g_bCallbackConnected = (iRes >= 0);
        } else {
          cpcGenericAnimation_WidgetConnect(deviceName, frontEnd);
        }
      } else {
        CPC_ProcessControlObject_WidgetDisconnection(g_sWidgetType);
      }
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      lockDPEs = cpcGenericAnimation_WidgetLockDPEs(deviceName);
      if (dynlen(lockDPEs) > 0) {
        if (g_bUnSystemAlarmPlc) {
          iRes = dpDisconnect("cpcGenericAnimation_WidgetLockCallBack", lockDPEs);
        }
        if (g_bDisconnectCB) {
          cpcGenericAnimation_WidgetDisconnect(deviceName, frontEnd);
        }
        g_bCallbackConnected = !(iRes >= 0);
        g_bDisconnectCB = false;
      } else {
        cpcGenericAnimation_WidgetDisconnect(deviceName, frontEnd);
      }
      CPC_ProcessControlObject_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_NOTHING:
      break;
    default:
      break;
  }
}

/** Init static values which are used in widget animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_ProcessControlObject_WidgetInitStatics(string deviceName)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
  dyn_string exceptionInfo;
  int iAlertType1, iAlertType2, iAlertType3, iAlertType4;
  int iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
                   deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
                   deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
                   deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
  g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));
  g_bMRestart = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
}

/** Returns the list of ProcessControlObject DPEs which should be connected on widget animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_ProcessControlObject_WidgetDPEs(string deviceName, dyn_string &dpes)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");
  dynAppend(dpes, deviceName + ".ProcessInput.OpMoSt");
  dynAppend(dpes, deviceName + ".ProcessInput.OpMoSt:_online.._invalid");
  dynAppend(dpes, deviceName + ".ProcessInput.AuOpMo:_online.._invalid");
  if (g_bBoolAlert) {
    dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");
    dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");
    dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state");
    dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");
  }
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names 
@param values input, the dpe values

*/
CPC_ProcessControlObject_WidgetAnimation(dyn_string dpes, dyn_anytype values, string widgetType)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
  int iSystemIntegrityAlarmValue = values[1];
  bool bSystemIntegrityAlarmEnabled = values[2];
  bool bLocked = values[3];
  string sSelectedManager = values[4];
  int iMask = values[5];

  bit32 bit32StsReg01 = values[6];
  bool bStsReg01Invalid = values[7];
  bit32 bit32StsReg02 = values[8];
  bool bStsReg02Invalid = values[9];
  int iOpMoSt = values[10];
  bool bOpMoStInvalid = values[11];
  bool bAuOpMoInvalid = values[12];
  int iStartIStState, iTStopIStState, iFuStopIStState, iAlSt;
  if (g_bBoolAlert) {
    iStartIStState = values[13];
    iTStopIStState = values[14];
    iFuStopIStState = values[15];
    iAlSt = values[16];
  }

  string deviceName, sFunction, blockColor, blockLetter, pcoColor, pcoName, sOpMo, displayColor;
  string sColorSelect, sWarningLetter, sWarningColor, sAlarmLetter, sAlarmColor, sControlLetter, sControlColor, sBodyColor;
  bool bLockVisible, isRestartNeeded;
  dyn_string dsFunctions, exceptionInfo;

  deviceName = unGenericDpFunctions_getDpName(dpes[6]);
  isRestartNeeded = g_bMRestart && getBit(bit32StsReg02, CPC_STSREG02_NEEDRESTART) == 0;
// 1. Select
  unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sColorSelect, bLockVisible);
// 2. Warning (Include the second stsReg: stsReg02 to check the warning letter)
  unGenericObject_WidgetWarningAnimationDoubleStsReg(bit32StsReg01, bit32StsReg02,
            makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
            makeDynInt(UN_STSREG02_ALBW),
            makeDynString(CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR),
            makeDynString(CPC_WIDGET_TEXT_WARNING_BLOCKED),
            sWarningLetter, sWarningColor);
// 2.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)
// or the checking is not enabled then set the letter to O and the color to not valid data

  if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
    {
    sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
    sWarningColor = "unDataNotValid";
    }

  if (bStsReg01Invalid || bStsReg02Invalid || bOpMoStInvalid || bAuOpMoInvalid)
    {
    sWarningLetter = CPC_WIDGET_TEXT_INVALID;
    sWarningColor = "unDataNotValid";
    }
  if (!bStsReg01Invalid && !bStsReg02Invalid) {
    // 3. Alarm
    unGenericObject_WidgetAlarmTextAnimation(bit32StsReg01,
                        makeDynInt(CPC_STSREG01_ALST, UN_STSREG01_STARTIST, CPC_STSREG01_TSTOPIST, CPC_STSREG01_FUSTOPIST),
                        makeDynString(CPC_WIDGET_TEXT_ALARM_ALST, CPC_WIDGET_TEXT_ALARM_STARTIST, CPC_WIDGET_TEXT_ALARM_TSTOPIST, CPC_WIDGET_TEXT_ALARM_FUSTOPIST),
                        sAlarmLetter, sAlarmColor);
    if (sAlarmLetter == "" && isRestartNeeded) {
      sAlarmLetter = CPC_WIDGET_TEXT_MANUAL_RESTART;
      sAlarmColor = "unFaceplate_WarningActive";
    }
    // 4. Control state
    cpcGenericObject_WidgetControlStateAnimation(bit32StsReg01, bit32StsReg02, makeDynString("CPC_STSREG02_SOFTLDST", "CPC_STSREG01_AUIHFOMOST", "UN_STSREG01_AUINHMMO", "UN_STSREG01_MMOST", "UN_STSREG01_FOMOST"), sControlLetter, sControlColor);
    // 5. Block text
    if (getBit(bit32StsReg02, UN_STSREG02_TFUSTOP) == 1) {
      blockLetter = CPC_WIDGET_TEXT_BLOCK_TFUSTOP;
      blockColor = "unFaceplate_StatusAlarmActive";
    }
    if (getBit(bit32StsReg02, UN_STSREG02_ALACTB) == 1) {
      blockLetter = CPC_WIDGET_TEXT_ALARM_BLOCKED;
      blockColor = "unFaceplate_StatusAlarmActive";
    }
  }
// 6. Animate letters and select
  if(g_bSystemConnected) {
    setMultiValue("WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
            "AlarmText", "text", sAlarmLetter, "AlarmText", "foreCol", sAlarmColor,
            "ControlStateText", "text", sControlLetter, "ControlStateText", "foreCol", sControlColor,
            "SelectArea", "foreCol", sColorSelect, "LockBmp", "visible", bLockVisible,
            "BlockText", "text", blockLetter, "BlockText", "foreCol", blockColor,
            "WidgetArea", "visible", true);
  }
// 7. Animate body
  sBodyColor = "unWidget_ControlStateAuto";
  if (bStsReg01Invalid || bStsReg02Invalid) {
    sBodyColor = "unDataNotValid";
  } else {
    if (sControlLetter == CPC_WIDGET_TEXT_CONTROL_FORCED) {
      sBodyColor = "unWidget_ControlStateForced";
    }
    if (isRestartNeeded) {
      sBodyColor = "unFaceplate_WarningActive";
    }
    if (sAlarmLetter == CPC_WIDGET_TEXT_ALARM_TSTOPIST || sAlarmLetter == CPC_WIDGET_TEXT_ALARM_FUSTOPIST) {
      sBodyColor = "unFaceplate_AlarmActive";
    }
    if (getBit(bit32StsReg01, UN_STSREG01_ALUNACK) == 1 && (iTStopIStState == 1 || iTStopIStState == 3 || iFuStopIStState == 1 || iFuStopIStState == 3)) {
      sBodyColor = "unWidget_AlarmNotAck";
    }

    if (blockLetter == CPC_WIDGET_TEXT_ALARM_BLOCKED) {
      sBodyColor = "unFaceplate_StatusAlarmActive";
    }
  }
    sFunction = cpcGenericObject_GetAnimationFunction(g_sWidgetType);
  if (isFunctionDefined(sFunction) && (sFunction != "")) {
    if(g_bSystemConnected)
      execScript("main(int iOn, int iOff, string sBodyColor) {" +
            sFunction + "(iOn, iOff, sBodyColor);}",
            makeDynString(), getBit(bit32StsReg01, UN_STSREG01_ONST), getBit(bit32StsReg01, UN_STSREG01_OFFST), sBodyColor);
  }
// 8. PCO name
  pcoColor = sBodyColor;
  if (getBit(bit32StsReg01, UN_STSREG01_OFFST) == 0 && g_sWidgetType != "Air") {
    pcoColor = "unWidget_AlternativeColor";
  }
  if (bStsReg01Invalid || bStsReg02Invalid) {
    pcoColor = "unDataNotValid";
  }
  CPC_ProcessControlObject_getName(deviceName, pcoName);
  if(g_bSystemConnected)
    setMultiValue("PCOName", "text", pcoName, "PCOName", "foreCol", pcoColor);
// 9. Option mode
  CPC_ProcessControlObject_getOptionModesLabel(deviceName, iOpMoSt, sOpMo);
  displayColor = "cpcColor_Widget_Request";
  if (bOpMoStInvalid) {
    displayColor = "unDataNotValid";
  }
  if(g_bSystemConnected) {
    setMultiValue("OptionMode", "text", sOpMo, "OptionMode", "foreCol", displayColor);
  }

// display event state
  if(g_bSystemConnected) {
    eventState.visible = iMask == 0;
  }
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}

/** Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_ProcessControlObject_WidgetDisconnection(string sWidgetType)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
  cpcGenericObject_WidgetDisconnection(sWidgetType, UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME);
  unGenericObject_WidgetDisplayValueDisconnect(makeDynString("OptionMode", "PCOName"));
  setValue("BlockText", "text", "");

  // set event state
  eventState.visible = false;
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}

/** Return button configuration including access level
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
mapping CPC_ProcessControlObject_ButtonConfig(string deviceName)
{
  mapping buttons;
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
  buttons[UN_FACEPLATE_BUTTON_SELECT] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[UN_FACEPLATE_BUTTON_AUTO_MODE] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[UN_FACEPLATE_BUTTON_MANUAL_MODE] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[UN_FACEPLATE_BUTTON_ONOPEN_REQUEST] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[UN_FACEPLATE_BUTTON_ACK_ALARM] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[UN_FACEPLATE_BUTTON_OPTION_MODES] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[UN_FACEPLATE_BUTTON_MANUAL_AUTO_TO_DEPENDENT_OBJECT] = CPC_ACCESS_RIGHTS_OPERATOR;
  buttons[CPC_FACEPLATE_BUTTON_ALLOW_RESTART] = CPC_ACCESS_RIGHTS_OPERATOR;

  buttons[UN_FACEPLATE_BUTTON_FORCED_MODE] = CPC_ACCESS_RIGHTS_EXPERT;
  buttons[UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION] = CPC_ACCESS_RIGHTS_EXPERT;
  buttons[UN_FACEPLATE_BUTTON_DEBLOCK_ALARM_ACTION] = CPC_ACCESS_RIGHTS_EXPERT;
  buttons[UN_FACEPLATE_BUTTON_SET_TEMPORARY_AS_FULL_STOP] = CPC_ACCESS_RIGHTS_EXPERT;
  buttons[UN_FACEPLATE_BUTTON_RESET_TEMPORARY_AS_FULL_STOP] = CPC_ACCESS_RIGHTS_EXPERT;
  buttons[UN_FACEPLATE_MASKEVENT] = CPC_ACCESS_RIGHTS_EXPERT;
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
  return buttons;
}

/** Configure the list of dpes that needs for buttons animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the name of device
@param dpes input, the list of dpes to connect
*/
CPC_ProcessControlObject_ButtonDPEs(string deviceName, dyn_string &dpes)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
}

/** Set the state of the contextual button of the device
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param dpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
CPC_ProcessControlObject_ButtonSetState(string deviceName, string dpType, dyn_string dsUserAccess, dyn_string dsData)
{
  bool bSelected, buttonEnabled, bit1, bit2, bit3;
  string localManager;
  dyn_string exceptionInfo;

  bool bLocked = dsData[1];
  string sSelectedUIManager = dsData[2];
  
  localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
  bSelected = (localManager == "S"); // Selection state
  dyn_string dsButtons = cpcGenericAnimation_ButtonList(deviceName);

//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
  mixed stsReg01Value = dsData[3];
  bool bStsReg01Bad = dsData[4];
  mixed stsReg02Value = dsData[5];
  bool bStsReg02Bad = dsData[6];
  for(int i = 1; i <= dynlen(dsButtons); i++) {
      buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
      switch (dsButtons[i]) {
        case UN_FACEPLATE_BUTTON_SET_TEMPORARY_AS_FULL_STOP:
          buttonEnabled = buttonEnabled && bSelected &&
            !getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
            !getBit(stsReg02Value, UN_STSREG02_TFUSTOP) &&
            !getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
            !bStsReg01Bad && !bStsReg02Bad;
          break;
        case UN_FACEPLATE_BUTTON_RESET_TEMPORARY_AS_FULL_STOP:
          buttonEnabled = buttonEnabled && bSelected &&
            !getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
            getBit(stsReg02Value, UN_STSREG02_TFUSTOP) &&
            !getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
            !bStsReg01Bad && !bStsReg02Bad;
          break;
        case UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION:
          buttonEnabled = buttonEnabled && bSelected &&
            !getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
            !getBit(stsReg02Value, UN_STSREG02_ALACTB) &&
            !getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
            !bStsReg01Bad && !bStsReg02Bad;
          break;
        case UN_FACEPLATE_BUTTON_DEBLOCK_ALARM_ACTION:
          buttonEnabled = buttonEnabled && bSelected &&
            !getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
            getBit(stsReg02Value, UN_STSREG02_ALACTB) &&
            !getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
            !bStsReg01Bad && !bStsReg02Bad;
          break;
        case UN_FACEPLATE_BUTTON_FORCED_MODE:
          buttonEnabled = buttonEnabled && bSelected &&
            !getBit(stsReg01Value, UN_STSREG01_FOMOST) &&
            !getBit(stsReg01Value, UN_STSREG01_AUINHFMO) &&
            !bStsReg01Bad && !bStsReg02Bad;
          break;
        case UN_FACEPLATE_BUTTON_MANUAL_MODE:
          buttonEnabled = buttonEnabled && bSelected &&
            !getBit(stsReg01Value, UN_STSREG01_MMOST) &&
            !getBit(stsReg01Value, UN_STSREG01_AUINHMMO) &&
            !getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
            !bStsReg01Bad && !bStsReg02Bad;
          break;
        case UN_FACEPLATE_BUTTON_AUTO_MODE:
        case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
        case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
        case UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST:
        case UN_FACEPLATE_BUTTON_OPTION_MODES:
        case UN_FACEPLATE_BUTTON_MANUAL_AUTO_TO_DEPENDENT_OBJECT:
          buttonEnabled = buttonEnabled && bSelected &&
            !getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
            !getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
            !bStsReg01Bad && !bStsReg02Bad;
          break;
        case UN_FACEPLATE_BUTTON_ACK_ALARM:
          buttonEnabled = buttonEnabled &&
            !getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
            !bStsReg01Bad && !bStsReg02Bad;
        case UN_FACEPLATE_BUTTON_NEXT:
        case UN_FACEPLATE_BUTTON_BACK:
        default:
          break;
      }

      if (dsButtons[i] == UN_FACEPLATE_BUTTON_SELECT) {
        unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
      } else if (dsButtons[i] == CPC_FACEPLATE_BUTTON_ALLOW_RESTART) {
        cpcButton_ButtonSetState(makeDynString(CPC_FACEPLATE_BUTTON_ALLOW_RESTART), dsUserAccess, deviceName, dpType, dsData, exceptionInfo);
      } else {
        cpcButton_setButtonState(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], buttonEnabled);
      }
  }
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
}

/** Init static values which are used in faceplate animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_ProcessControlObject_FaceplateInitStatics(string deviceName)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
  int iRes, iAlertType1, iAlertType2, iAlertType3, iAlertType4, iRangeType;
  dyn_string exceptionInfo;
  iRes = dpGet(deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType1,
               deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType2,
               deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType3,
               deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
  g_params["BoolAlert"] = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));

  dyn_string exceptionInfo;
  g_params["MRestart"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
}

/** Returns the list of ProcessControlObject DPEs which should be connected on faceplate animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_ProcessControlObject_FaceplateDPEs(string deviceName, dyn_string &dpes)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");
  dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");
  dynAppend(dpes, deviceName + ".ProcessInput.OpMoSt");
  dynAppend(dpes, deviceName + ".ProcessInput.OpMoSt:_online.._invalid");
  dynAppend(dpes, deviceName + ".ProcessInput.AuOpMo");
  dynAppend(dpes, deviceName + ".ProcessInput.AuOpMo:_online.._invalid");
  if (g_params["BoolAlert"]) {
    dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");
    dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");
    dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state");
    dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");
  }
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
}

/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names 
@param values input, the dpe values

*/
CPC_ProcessControlObject_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_ONST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_OFFST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG02_ALACTB", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG02_TFUSTOP", true, "cpcColor_Faceplate_Status");

  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG02_RUNOST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG02_CSTOPOST", true, "cpcColor_Faceplate_Status");
  CPC_ProcessControlObject_animateOptionMode(dpes, values, "OpMoSt", "cpcColor_Faceplate_Request");

  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_AUMOST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_MMOST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_AUINHMMO", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_FOMOST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_AUINHFMO", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "CPC_STSREG02_SOFTLDST", true, "cpcColor_Faceplate_Status");

  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG02_AUONRST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG02_MONRST", true, "cpcColor_Faceplate_Status");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG02_AUCSTOP", true, "cpcColor_Faceplate_Status");
  CPC_ProcessControlObject_animateOptionMode(dpes, values, "AuOpMo", "cpcColor_Faceplate_Request");

  cpcFaceplate_animateInterlock(dpes, values, "UN_STSREG01_STARTIST",   "StartISt",  "UN_STSREG01_ALUNACK");
  cpcFaceplate_animateInterlock(dpes, values, "CPC_STSREG01_TSTOPIST",  "TStopISt",  "UN_STSREG01_ALUNACK");
  cpcFaceplate_animateInterlock(dpes, values, "CPC_STSREG01_FUSTOPIST", "FuStopISt", "UN_STSREG01_ALUNACK");
  cpcFaceplate_animateInterlock(dpes, values, "CPC_STSREG01_ALST",      "AlSt",      "UN_STSREG01_ALUNACK");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_ALUNACK", true, "cpcColor_Alarm_Bad");

  cpcFaceplate_animateMaskEvent(dpes, values);
  cpcFaceplate_animateNeedRestart(dpes, values);
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_IOERRORW", true, "unFaceplate_WarningActive");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_IOSIMUW", true, "unFaceplate_WarningActive");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG02_ALBW", true, "unFaceplate_WarningActive");
  cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_AUMRW", true, "unFaceplate_WarningActive");
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
}


//begin_TAG_SCRIPT_DEVICE_TYPE_CustomFunctions
/**Animate option mode in faceplate.

Function is alike cpcFaceplate functions.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
@param dpeName name of parameter to animate
@param color field color on valid state
*/
CPC_ProcessControlObject_animateOptionMode(dyn_string dpes, dyn_string values, string dpeName, string color)
{
  string convertedValue;
  int value;
  bool invalid;
  if (dynlen(dpes) > 0) { // online animation
    cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, dpeName, value, invalid);
    CPC_ProcessControlObject_getOptionModesLabel(unGenericDpFunctions_getDpName(dpes[6]), value, convertedValue);
    cpcGenericObject_DisplayText(convertedValue, dpeName, color,
        invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
  } else {
    unGenericObject_DisplayValueDisconnect(dpeName);
  }
}

/**
Purpose: this function is called by a PCO object to get an option mode label

Parameter:
	-> sDpName, string input, device name
	-> uLabel, unsigned input, label number
	-> result, string output, result of the function

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_getOptionModesLabel(string sDpName, unsigned uLabel, string &result)
{
	string sOptionModes;
	dyn_string dsOptionModes;

	result = "";
  //todo: description should be fetched in advance, not in animation function!
	sOptionModes = unGenericDpFunctions_getDescription(sDpName + UN_PCO_OPTION_MODE_PATH);
	dsOptionModes = strsplit(sOptionModes, UN_PCO_OPTION_MODE_DELIMITER);
	if ((dynlen(dsOptionModes) >= uLabel) && (uLabel > 0) && (UN_PCO_OPTION_MODE_NUMBER >= uLabel))
		{
		result = dsOptionModes[uLabel];
		}
	if (result == "")
		{
		//result = "Op Mode " + uLabel;
		}
}

/**
Purpose: this function is called by a PCO object to get an option mode allowance

Parameter:
	-> sDpName, string input, device name
	-> uLabel, unsigned input, label number
	-> result, bit32 output, result of the function

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_getOptionModesAllowance(string sDpName, unsigned uLabel, bit32 &result)
{
	string sOptionModes;
	dyn_string dsOptionModes;

	result = 0;
  //todo: description should be fetched in advance, not in animation function!
	sOptionModes = unGenericDpFunctions_getDescription(sDpName + UN_PCO_OPTION_MODE_PATH);
	dsOptionModes = strsplit(sOptionModes, UN_PCO_OPTION_MODE_DELIMITER);
	if ((dynlen(dsOptionModes) >= (UN_PCO_OPTION_MODE_NUMBER + uLabel)) && (uLabel > 0))
		{
		result = dsOptionModes[UN_PCO_OPTION_MODE_NUMBER + uLabel];
		}
}

/**
Purpose: this function is called by a PCO object to set the option mode labels & allowance

Parameter:
	-> sDpName, string input, device name
	-> dsLabels, dyn_string input, labels
	-> dsAllowances, dyn_string input, allowances (must be bit32 variables converted to string)
	-> iRes, int output, result of the function, < 0 if an error occured

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_setOptionModes(string sDpName, dyn_string dsLabels, dyn_string dsAllowances, int &iRes)
{
	int i,length;
	string description;

	iRes = -1;
	length = dynlen(dsLabels);
	if ((length != dynlen(dsAllowances)) || (length == 0))
		{
		//error
		}
	else
		{
		for(i=1;i<=UN_PCO_OPTION_MODE_NUMBER;i++)			// Labels
			{
			if (i <= length)
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER + dsLabels[i];
				}
			else
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER;
				}
			}
		for(i=1;i<=UN_PCO_OPTION_MODE_NUMBER;i++)			// Allowances
			{
			if (i <= length)
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER + dsAllowances[i];
				}
			else
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER;
				}
			}
		description = substr(description, 1);
		iRes = dpSetDescription(sDpName + UN_PCO_OPTION_MODE_PATH, description);
		}
}

/**
Purpose: this function is called by a PCO object to get PCO name

Parameter:
	-> sDpName, string input, device name
	-> result, string output, result of the function

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_getName(string sDpName, string &result)
{
	dyn_string dsParameters;

	unGenericDpFunctions_getParameters(sDpName, dsParameters);
	result = dsParameters[UN_PARAMETER_NAME];
	if (result == "")
		{
		result = "PCO";
		}
}

/**
Purpose: this function is called by a PCO object to set the PCO name

Parameter:
	-> sDpName, string input, device name
	-> name, string input, name to be set
	-> exceptionInfo, dyn_string output, for errors

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_setName(string sDpName, string name, dyn_string &exceptionInfo)
{
	unGenericDpFunctions_changeDpParameter(sDpName, UN_PARAMETER_NAME, name, exceptionInfo);
}

// needs for backward compatibility as unicos core uses this function in Device configuration panel
unPCO_setName(string sDpName, string name, dyn_string &exceptionInfo)
{
  CPC_ProcessControlObject_setName(sDpName, name, exceptionInfo);
}
unPCO_getNextAllowedModes(string sDpName, int iCurrentMode, bool bRunning, dyn_string &dsAllowedModes, dyn_string &exceptionInfo)
{
	string parameters;
	dyn_string dsParameters;
	bit32 allowance;
	int i;

	dynClear(dsAllowedModes);
/*	if ((iCurrentMode <= 0) || (iCurrentMode >  UN_PCO_OPTION_MODE_NUMBER))
		{
		fwException_raise(exceptionInfo,"ERROR","unPCO_getNextAllowedModes : " + getCatStr("unFunctions","BADINPUTS"),"");
		return;
		}
*/
	// 1. Get parameters
	parameters = unGenericDpFunctions_getDescription(sDpName + UN_PCO_OPTION_MODE_PATH);
	dsParameters = strsplit(parameters, UN_PCO_OPTION_MODE_DELIMITER);
	while (dynlen(dsParameters) < (2*UN_PCO_OPTION_MODE_NUMBER))
		{
		dynAppend(dsParameters, "");
		}

	// 2. Not running -> each defined mode is allowed
	if (bRunning == false)
		{
		for(i=1;i<=UN_PCO_OPTION_MODE_NUMBER;i++)
			{
			if ((strltrim(dsParameters[i]) != "") && (i != iCurrentMode))
				{
				dynAppend(dsAllowedModes,dsParameters[i]);
				}
			}
		}
	else
		{

	// 3. While running -> allowed modes are coded in the allowance table
		allowance = dsParameters[iCurrentMode+UN_PCO_OPTION_MODE_NUMBER];
		for(i=1;i<=UN_PCO_OPTION_MODE_NUMBER;i++)
			{
			if ((strltrim(dsParameters[i]) != "") && (getBit(allowance,UN_PCO_OPTION_MODE_NUMBER-i) == 1) && (i != iCurrentMode))
				{
				dynAppend(dsAllowedModes,dsParameters[i]);
				}
			}
		}
}
unPCO_getOptionModesAllowance(string sDpName, unsigned uLabel, bit32 &result)
{
  CPC_ProcessControlObject_getOptionModesAllowance(sDpName, uLabel, result);
}
unPCO_setOptionModes(string sDpName, dyn_string dsLabels, dyn_string dsAllowances, int &iRes)
{
  CPC_ProcessControlObject_setOptionModes(sDpName, dsLabels, dsAllowances, iRes);
}



/**
Purpose: to get the modes you can access from current mode (using allowance tables)

Parameter:
	-> sDpName, string input, device name
	-> iCurrentMode, int input, current mode
	-> bRunning, bool input, true while running
	-> dsAllowedModes, dyn_string output, result of the function
	-> exceptionInfo, dyn_string output, for errors

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_getNextAllowedModes(string sDpName, int iCurrentMode, bool bRunning, dyn_string &dsAllowedModes, dyn_string &dsUnAllowedModes, dyn_string &exceptionInfo)
{
	string parameters;
	dyn_string dsParameters;
	bit32 allowance;

	dynClear(dsAllowedModes);
	dynClear(dsUnAllowedModes);

	// 1. Get parameters
	parameters = unGenericDpFunctions_getDescription(sDpName + UN_PCO_OPTION_MODE_PATH);
	dsParameters = strsplit(parameters, UN_PCO_OPTION_MODE_DELIMITER);
	while (dynlen(dsParameters) < (2*UN_PCO_OPTION_MODE_NUMBER)) {
		dynAppend(dsParameters, "");
	}
	if (bRunning) {
		// 3. While running -> allowed modes are coded in the allowance table
		allowance = dsParameters[iCurrentMode+UN_PCO_OPTION_MODE_NUMBER];
		for(int i = 1; i <= UN_PCO_OPTION_MODE_NUMBER; i++) {
			if (strltrim(dsParameters[i]) != "") {
				if ((getBit(allowance,UN_PCO_OPTION_MODE_NUMBER-i) == 1) && (i != iCurrentMode)) {
					dynAppend(dsAllowedModes,dsParameters[i]);
				} else {
					dynAppend(dsUnAllowedModes,dsParameters[i]);
				}
			}
		}
	} else {
		// 2. Not running -> each defined mode is allowed
		for(int i = 1; i <= UN_PCO_OPTION_MODE_NUMBER; i++) {
			if (strltrim(dsParameters[i]) != "") {
				if (i != iCurrentMode) {
					dynAppend(dsAllowedModes, dsParameters[i]);
				} else {
					dynAppend(dsUnAllowedModes, dsParameters[i]);
				}
			}
		}
	}
}
//end_TAG_SCRIPT_DEVICE_TYPE_CustomFunctions


//@}