/**@file

// cpcWordParameterConfig.ctl
This library contains the import and export function of the CPC_WordParameter.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{



// Constants
const string UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME = "CPC_WordParameter";
//begin_TAG_SCRIPT_DEVICE_TYPE_constants
const unsigned UN_CONFIG_CPC_WORDPARAMETER_LENGTH				= 15;

const unsigned UN_CONFIG_CPC_WORDPARAMETER_UNIT					= 1;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_FORMAT				= 2;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_DEADBAND				= 3;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_DEADBAND_TYPE		= 4;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_RANGEMAX				= 5;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_RANGEMIN				= 6;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ARCHIVE_ACTIVE		= 7;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ARCHIVE_TIME_FILTER	= 8;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_MPOSRST		= 9;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_POSST		= 10;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_MPOSR		= 11;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_STSREG01		= 12;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_EVSTSREG01	= 13;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_MANREG01		= 14;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_DEFAULT_VALUE		= 15;

const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_LENGTH          = 7;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_DESCRIPTION     = 2;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_MASTER_NAME     = 3;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_PARENTS         = 4;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_CHILDREN        = 5;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_TYPE            = 6;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_SECOND_ALIAS    = 7;
//end_TAG_SCRIPT_DEVICE_TYPE_constants


/**
DPE configuration
TODO: use fw info instead
*/
mapping CPC_WordParameterConfig_getConfig() {
  mapping config;
  mapping props;

//begin_TAG_SCRIPT_DEVICE_TYPE_deviceConfig
    mappingClear(props);
    props["address"] 		= ".ProcessInput.StsReg01";
    props["dataType"] 		= CPC_UINT16;
    config["StsReg01"] 		= props;

    mappingClear(props);
    props["hasArchive"] 	= true;
    props["address"] 		= ".ProcessInput.evStsReg01";
    props["dataType"] 		= CPC_INT32;
    config["evStsReg01"] 	= props;

    mappingClear(props);
    props["hasUnit"] 		= true;
    props["hasFormat"]		= true;
    props["hasSmooth"] 		= true;
    props["address"] 		= ".ProcessInput.MPosRSt";
    props["dataType"] 		= CPC_UINT16;
    config["MPosRSt"] 		= props;

    mappingClear(props);
    props["hasUnit"] 		= true;
    props["hasFormat"]		= true;
    props["hasSmooth"] 		= true;
    props["hasArchive"] 	        = true;
    props["hasPvRange"] 	        = true;
    props["hasDescription"]         = true;
    props["address"] 		= ".ProcessInput.PosSt";
    props["dataType"] 		= CPC_UINT16;
    config["PosSt"] 		= props;

    mappingClear(props);
    props["address"] 		= ".ProcessOutput.ManReg01";
    props["dataType"] 		= CPC_UINT16;
    config["ManReg01"] 		= props;

    mappingClear(props);
    props["hasUnit"] 		= true;
    props["hasFormat"]		= true;
    props["address"] 		= ".ProcessOutput.MPosR";
    props["dataType"] 		= CPC_UINT16;
    config["MPosR"] 		= props;
//end_TAG_SCRIPT_DEVICE_TYPE_deviceConfig

  return config;
}

/**
Check data before configuration
  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_WordParameterConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH) {
    // 1. Common
    unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, exceptionInfo);
    // 2. call the device check function
    unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, exceptionInfo);
  } else {
    fwException_raise(exceptionInfo,"ERROR","CPC_WordParameterConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}

/**
Check the CPC_WordParameter device configuration for the S7_PLC front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
S7_PLC_CPC_WordParameter_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, configLine, exceptionInfo);
}

/**
Check the CPC_WordParameter device configuration for the _UnPlc front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
_UnPlc_CPC_WordParameter_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, configLine, exceptionInfo);
}

/** Check custom configuration
*/
CPC_WordParameterConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
	int iValue, iMax, iMin;
	unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_DEFAULT_VALUE], iValue, exceptionInfo);
	unConfigGenericFunctions_checkFloat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_RANGEMIN], iMin, exceptionInfo);
	unConfigGenericFunctions_checkFloat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_RANGEMAX], iMax, exceptionInfo);
	if((iValue < iMin) || (iValue > iMax)) {
		fwException_raise(exceptionInfo,"ERROR","CPC_WordParameterConfig_checkCommonConfig: " + getCatStr("unCPCGeneration","BADRANGE"),"");	
	}
//end_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
}

/**
Creates dp (or checks if it is already presented)and sets configuration for CPC_WordParameter object.
  
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_WordParameterConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;

  // 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, exceptionInfo);
  if (dynlen(exceptionInfo) <= 0) { // Good inputs 
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, exceptionInfo);
    if (dynlen(exceptionInfo) <= 0) { // Creation ok or dp already exists
      // 2. Generate device
      // 2.1. Parameters
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
      // 2.2 call the device check function
      unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, exceptionInfo);
    }
  }
}

/**
Set the CPC_WordParameter device configuration for the S7_PLC front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
S7_PLC_CPC_WordParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

/**
Set the CPC_WordParameter device configuration for the _UnPlc front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
_UnPlc_CPC_WordParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

/** Set custom configuration
*/
CPC_WordParameterConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
  unConfigGenericFunctions_setUnit(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_UNIT], exceptionInfo);
  unConfigGenericFunctions_setFormat(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_FORMAT], exceptionInfo);
  dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_DEFAULT_VALUE]);
//end_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
}

/**
Purpose: Export CPC_WordParameter Devices

Usage: External function

PVSS manager usage: NG, NV
*/
CPC_WordParameterConfig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
//begin_TAG_SCRIPT_DEVICE_TYPE_export
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosRSt", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosSt",		true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosR", 		false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_export

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

/**
Purpose: Export CPC_WordParameter Devices for S7_PLC front-end

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_WordParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    CPC_WordParameterConfig_ExportConfig(dsDpList, exceptionInfo);
}

/**
Purpose: Export CPC_WordParameter Devices for _UnPlc front-end

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_WordParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    CPC_WordParameterConfig_ExportConfig(dsDpList, exceptionInfo);
}

/*
  Use this place to define any unexisting constants or functions than you need for the device implementation.
*/
//begin_TAG_SCRIPT_DEVICE_TYPE_CustomConfigFunctions
bool CPC_WordParameterConfig_isWithText(string posStDescription)
{
  posStDescription = strrtrim(posStDescription);
  return posStDescription != "" && posStDescription != CPC_CONFIG_EMPTY_DESCRIPTION_STRING;
}
//end_TAG_SCRIPT_DEVICE_TYPE_CustomConfigFunctions


//@}