/**@file

// cpcWordStatus.ctl
This library contains the widget, faceplate, etc. functions of WordStatus.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{


/** Function called from snapshot utility of the treeDeviceOverview to get the time and value
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name
@param deviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
CPC_WordStatus_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
  CPC_AnalogStatus_ObjectListGetValueTime(deviceName, deviceType, dsReturnData);
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}

/** pop-up menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param dsAccessOk input, the access control 
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
CPC_WordStatus_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
  dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_DIAG_INFO_TEXT, UN_POPUPMENU_TREND_TEXT);

  cpcGenericObject_addDefaultUnicosActionToMenu(deviceName, dpType, dsMenuConfig, dsAccessOk, menuList);
  cpcGenericObject_addTrendActionToMenu(deviceName, dpType, dsMenuConfig, dsAccessOk, menuList);
//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** handle the answer of the popup menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control 
@param menuAnswer input, selected menu value 
*/
CPC_WordStatus_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer)
{
  dyn_string exceptionInfo;

//begin_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
  cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_HandleMenu

  if (dynlen(exceptionInfo) > 0) {
      unSendMessage_toExpertException("Right Click",exceptionInfo);
  }
}

/**  widget register callback

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
CPC_WordStatus_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, systemName, frontEnd;
  int iAction, iRes;
  dyn_string exceptionInfo;
  bool bRemote;

  deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
  if (deviceName == "") deviceName = g_sDpName; // In case of disconnection
  systemName = unGenericDpFunctions_getSystemName(deviceName);
  frontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
  if (frontEnd != "") frontEnd = systemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+frontEnd;

  unDistributedControl_isRemote(bRemote, systemName);
  g_bSystemConnected = bRemote ? bSystemConnected : true;
  g_bUnSystemAlarmPlc = frontEnd != "" && dpExists(frontEnd);
  if (deviceName != "") {
    if (dpTypeName(deviceName) != UN_CONFIG_CPC_WORDSTATUS_DPT_NAME) g_bUnSystemAlarmPlc = false;
  }

  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  dyn_string lockDPEs;
  switch (iAction) {
    case UN_ACTION_DISCONNECT:
      CPC_WordStatus_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_DPCONNECT:
      g_sDpName = deviceName;
      if (g_bUnSystemAlarmPlc) {
        lockDPEs = cpcGenericAnimation_WidgetLockDPEs(deviceName);
        if (dynlen(lockDPEs) > 0) {
          iRes = dpConnect("cpcGenericAnimation_WidgetLockCallBack", lockDPEs);
          g_bCallbackConnected = (iRes >= 0);
        } else {
          cpcGenericAnimation_WidgetConnect(deviceName, frontEnd);
        }
      } else {
        CPC_WordStatus_WidgetDisconnection(g_sWidgetType);
      }
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      lockDPEs = cpcGenericAnimation_WidgetLockDPEs(deviceName);
      if (dynlen(lockDPEs) > 0) {
        if (g_bUnSystemAlarmPlc) {
          iRes = dpDisconnect("cpcGenericAnimation_WidgetLockCallBack", lockDPEs);
        }
        if (g_bDisconnectCB) {
          cpcGenericAnimation_WidgetDisconnect(deviceName, frontEnd);
        }
        g_bCallbackConnected = !(iRes >= 0);
        g_bDisconnectCB = false;
      } else {
        cpcGenericAnimation_WidgetDisconnect(deviceName, frontEnd);
      }
      CPC_WordStatus_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_NOTHING:
      break;
    default:
      break;
  }
}

/** Init static values which are used in widget animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_WordStatus_WidgetInitStatics(string deviceName)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
  string sDp = deviceName + ".ProcessInput.PosSt";
  g_sPosStFormat = dpGetFormat(sDp);
  g_sPosStUnit = dpGetUnit(sDp);
  cpcGenericObject_StringToMapping(dpGetDescription(sDp), g_mLabels);
  if (g_sWidgetType == "WordStatusBit" || g_sWidgetType == "WordStatusBitBig") { // fill with defaults
      for (int i = 0; i < 16; i++) {
          if (!mappingHasKey(g_mLabels, (string)i)) {
              g_mLabels[(string)i] = "N/A";
          }
      }

      dyn_string exceptionInfo;
      g_mLabels[CPC_PARAMS_NO_BIT]    = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_NO_BIT, exceptionInfo, "No Status");
      g_mLabels[CPC_PARAMS_MULTI_BIT] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MULTI_BIT, exceptionInfo, "Multiple Status");
  }
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
}

/** Returns the list of WordStatus DPEs which should be connected on widget animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_WordStatus_WidgetDPEs(string deviceName, dyn_string &dpes)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
  dynRemove(dpes, dynlen(dpes));
  dynAppend(dpes, deviceName + ".ProcessInput.PosSt");
  dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names 
@param values input, the dpe values

*/
CPC_WordStatus_WidgetAnimation(dyn_string dpes, dyn_anytype values, string widgetType)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
  int iSystemIntegrityAlarmValue = values[1];
  bool bSystemIntegrityAlarmEnabled = values[2];
  bool bLocked = values[3];
  string sSelectedManager = values[4];

  int iPosSt = values[5];
  bool bPosStInvalid = values[6];

  string sWarningLetter, sWarningColor;
  int iIndex, iLength, iBitCounter,iBitPos;
  string sBody1Text, sBody1Color = "cpcColor_Widget_Status";

  if (widgetType == "WordStatusBit") {
    for (int j = 0; j < 16; j++) {
      if (getBit(iPosSt, j) == 1) {
        if (iBitCounter == 0) {
          iBitPos = j;
        }
        iBitCounter = iBitCounter + 1;
      }
    }
    if (iBitCounter == 0) {
      sBody1Text = g_mLabels[CPC_PARAMS_NO_BIT];
    } else if (iBitCounter == 1) {
      sBody1Text = CPC_WordStatus_lookupLabel(g_mLabels, iBitPos);
    } else {
      sBody1Text = g_mLabels[CPC_PARAMS_MULTI_BIT];
    }
  } else {
    sBody1Text = mappingHasKey(g_mLabels, (string)iPosSt) ? g_mLabels[(string)iPosSt] : iPosSt + " " + g_sPosStUnit;
  }

  cpcGenericObject_WidgetValidnessAnimation(bSystemIntegrityAlarmEnabled, iSystemIntegrityAlarmValue,
                                            bPosStInvalid,
                                            sWarningLetter, sWarningColor);
  if (bPosStInvalid) {
    sBody1Color = "unDataNotValid";
  }

  if(g_bSystemConnected) {
        setMultiValue("WarningText",  "text",     sWarningLetter, "WarningText", "foreCol",  sWarningColor);
        if(shapeExists("WidgetArea"))
            setValue("WidgetArea", "visible", true);
  }
  
  if (widgetType == "WordStatusBitBig") {
    for (int j = 0; j < 16; j++) {
      string sTempBodyText = CPC_WordStatus_lookupLabel(g_mLabels, j);
      string bodyRef = "Body" + j;
      setMultiValue(bodyRef,        "text",     sTempBodyText,  bodyRef,       "fill",    "[solid]",
                    bodyRef,        "foreCol",  sBody1Color,    bodyRef,       "backCol", "unWidget_Background");
      unGenericObject_ColoredSquareAnimate("bit" + j, getBit(iPosSt, j), true, "cpcColor_Widget_Status", "cpcColor_Widget_Status", false);
    }
  } else {
        setMultiValue("Body1",        "text",     sBody1Text,     "Body1",       "fill",    "[solid]",
                      "Body1",        "foreCol",  sBody1Color,    "Body1",       "backCol", "unWidget_Background");
  }
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}

/** Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_WordStatus_WidgetDisconnection(string sWidgetType)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
  dyn_string dsSquareName=makeDynString("");
  setMultiValue("Body1", "foreCol", "unDataNoAccess", "Body1", "backCol", "unWidget_Background", "Body1", "fill", "[solid]", "WarningText", "text", "");
  if(shapeExists("WidgetArea"))
     setValue("WidgetArea", "visible", false);
  if(sWidgetType == "WordStatusBitBig") {
      for(int j = 0; j < mappinglen(g_mLabels); j++) {
          dynAppend(dsSquareName,"bit"+j);
          setValue("bit"+j +".border", "backCol", "unDataNoAccess");
          setValue("bit"+j +".body", "backCol", "unWidget_Background");
      }
  }
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}

/** Init static values which are used in faceplate animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_WordStatus_FaceplateInitStatics(string deviceName)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
  string sDP = deviceName + ".ProcessInput.PosSt";
  g_params["PosStFormat"] = dpGetFormat(sDP);
  g_params["PosStUnit"] = dpGetUnit(sDP);
  cpcGenericObject_StringToMapping(dpGetDescription(sDP), g_mLabels);
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
}

/** Returns the list of WordStatus DPEs which should be connected on faceplate animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_WordStatus_FaceplateDPEs(string deviceName, dyn_string &dpes)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
  dynRemove(dpes, dynlen(dpes));
  dynAppend(dpes, deviceName + ".statusInformation.selectedManager:_lock._original._locked");
  dynAppend(dpes, deviceName + ".statusInformation.selectedManager");
  dynAppend(dpes, deviceName + ".ProcessInput.PosSt");
  dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
}

/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names 
@param values input, the dpe values

*/
CPC_WordStatus_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
  int iSystemIntegrityAlarmValue = values[1];
  bool bSystemIntegrityAlarmEnabled = values[2];
  bool bLocked = values[3];
  string sSelectedManager = values[4];
  int iPosSt = values[5];
  bool bPosStInvalid = values[6];

  string sWarningLetter, sWarningColor;
  string body1Color = "unWidget_ControlStateAuto";

  cpcGenericObject_WidgetValidnessAnimation(bSystemIntegrityAlarmEnabled, iSystemIntegrityAlarmValue,
                                          bPosStInvalid,
                                          sWarningLetter, sWarningColor);

  if (bPosStInvalid) {
    body1Color = "unDataNotValid";
  }

  if (g_params["WidgetType"] == "WordStatusBit") {
    string body1Text = "";
    for (int j = 0; j < mappinglen(g_mLabels); j++) {
      string sTempBodyText = CPC_WordStatus_lookupLabel(g_mLabels, j);
      body1Text = body1Text + sTempBodyText + "\n";
      if (g_bSystemConnected) {
        unGenericObject_ColoredSquareAnimate("bit" + j, getBit(iPosSt, j), true, "unFaceplate_StatusActive", "unFaceplate_StatusActive", false);
      }
    }
    if (g_bSystemConnected)
      setMultiValue("Body1", "text", body1Text, "Body1", "fill", "[solid]",
                    "Body1", "foreCol", body1Color, "Body1", "backCol", "unWidget_Background",
                    "WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor);
  } else {
    string uniqueMessageText = mappingHasKey(g_mLabels, (string)iPosSt) ? g_mLabels[(string)iPosSt] : "No Description";
    string decimalValueText = iPosSt;
    if (g_bSystemConnected)
      setMultiValue("UniqueMessageText", "text", uniqueMessageText, "UniqueMessageText", "fill", "[solid]",
                    "UniqueMessageText", "foreCol", body1Color, "UniqueMessageText", "backCol", "unWidget_Background",
                    "DecimalValue", "text", decimalValueText, "DecimalValue", "fill", "[solid]",
                    "DecimalValue", "foreCol", body1Color, "DecimalValue", "backCol", "unWidget_Background",
                    "WarningText2", "text", sWarningLetter, "WarningText2", "foreCol", sWarningColor);
  }
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
}

/**Faceplate disconnection function.

Normally you should not use this function.
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
CPC_WordStatus_FaceplateStatusDisconnection()
{
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusDisconnection
  if (g_params["WidgetType"] == "WordStatusBit") {
    setMultiValue("Body1", "foreCol", "unDataNoAccess", "WarningText", "foreCol", "unDataNoAccess");
    for (int j = 0; j < mappinglen(g_mLabels); j++){
      setMultiValue("bit"+j +".border", "backCol", "unDataNoAccess",
              "bit"+j +".body",   "backCol", "unWidget_Background");
    }
  } else {
    setMultiValue("UniqueMessageText", "foreCol", "unDataNoAccess",
                  "DecimalValue",      "foreCol", "unDataNoAccess",
                  "WarningText2",      "foreCol", "unDataNoAccess");
  }
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusDisconnection
}


//begin_TAG_SCRIPT_DEVICE_TYPE_CustomFunctions
string CPC_WordStatus_lookupLabel(mapping labels, string idx)
{
    if (mappingHasKey(labels, idx)) {
        return labels[idx];
    } else {
        return "No message attributed";
    }
}
//end_TAG_SCRIPT_DEVICE_TYPE_CustomFunctions


//@}