/**@file

// cpcRcpBuffersGenericFunctions.ctl
This library contains the generic functions of the package.

@par Creation Date
  07/06/2011

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  Ivan Prieto Barreiro (EN-ICE)
*/

//@{

//------------------------------------------------------------------------------------------------------------------------
// cpcRcpBuffersGenericFunctions_addDefaultUser
/** add/reset default user, password not change if already set.
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
cpcRcpBuffersGenericFunctions_addDefaultUser()
{
  string acComponent="fwAccessControl";
  string acVersion="3.2.7", sPassword;
  int acInstalled, iPos;
  dyn_string exceptionInfo, dsUserName, dsPassword;
  bool bIntegratedMode;
  
  fwInstallation_componentInstalled(acComponent, acVersion, acInstalled);
  if (!acInstalled)
  {
    DebugTN("unGenericFunctions_setDefaultUser: Access Control component " + acVersion + " not installed!");
    return;
  }

  bIntegratedMode=_fwAccessControl_integratedMode();
  if(!bIntegratedMode)
  {
    // use here the fwAccessControl functions to create domains, privileges, groups, users
    DebugTN("unGenericFunctions_setDefaultUser: TAG_PREFIX domain, group and user updated");
  }
  else
    DebugTN("unGenericFunctions_setDefaultUser: fwAccessControl mode activated, the changes are automatically populated from the AC server.");
  
}

//------------------------------------------------------------------------------------------------------------------------
// cpcRcpBuffersGenericFunctions_FaceplateStatusHeader
/** set the text and the colors of the header in the faceplate status tab (state of the FE connection)
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sFEDp  input, front-end DP name
@param sFEAlias  input, front-end name (alias)
@param sDeviceAlias  input, device name (alias)
@param iAlarm  input, front-end comm system integrity alarm level
@param bSystemAlarmEnable  input, front-end comm system integrity enable/disable
@param bInvalid  input, device invalid state
*/
cpcRcpBuffersGenericFunctions_FaceplateStatusHeader(string sFEDp, string sFEAlias, string sDeviceAlias, int iAlarm, bool bSystemAlarmEnable, bool bInvalid)
{
  bool bOk=false, bFrontEnd=false;
  time tLast;
  string sBackColor, sForeColor, sText, sTime, sName;

  sName=sFEAlias;
  if (sName=="")
    sName="?";
  else if (sName!=sFEDp)
    bFrontEnd=true;

  if(bInvalid)
  {
    sBackColor="unDataNotValid";
    sForeColor="_3DText";
    sTime="";
    sText="DATA INVALID";
  }
  else if(!bSystemAlarmEnable)
  {
    //SystemAlarm disabled
    //sBackColor="unAlarmMasked";
    sBackColor="unDataNotValid";  
    sForeColor="_3DText";
    sTime="";
    sText=sName + " DATA Connection DISABLED";
  }
  else
  {
    if(iAlarm > c_unSystemIntegrity_no_alarm_value)
    {
      //Connection stopped
      sBackColor="unDataNotValid";
      sForeColor="_3DText";
      sText=sName + " NO DATA Connection";
   
      if(bFrontEnd)
      {
        if (dpExists(sFEDp))
        {
         dpGet(sFEDp + ".communication.counter:_online.._stime", tLast);
         sTime=" since: " + (string)tLast; 
        }
        else
        {
         sTime="?";
        }  
      }
      else
        sTime="";
    }
    else
    {
      //Connection FE OK
      sBackColor="_3DFace";
      sForeColor="_3DText";
      sText=sName + " DATA Connection OK";
      sTime="";   
      bOk=true;
    }
  }
  sText=sText + sTime;
  if(shapeExists("txtFEstatus"))
  {
    setMultiValue("txtFEstatus", "backCol", sBackColor,
                  "txtFEstatus", "foreCol", sForeColor,
                  "txtFEstatus", "text", sText);
  }
}

//------------------------------------------------------------------------------------------------------------------------
// cpcRcpBuffersGenericFunctions_FaceplateStatusDisconnect
/** set the text and the colors of the header in the faceplate status tab (state of the FE connection) in 
  case of disconnection of the remote system
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
void cpcRcpBuffersGenericFunctions_FaceplateStatusDisconnect()
{
  string sBackColor, sForeColor, sText;

  sBackColor="_3DFace";
  sForeColor="unDataNoAccess";
  sText="DATA not connected";
    
  if(shapeExists("txtFEstatus"))
  {
    setMultiValue("txtFEstatus", "backCol", sBackColor,
                  "txtFEstatus", "foreCol", sForeColor,
                  "txtFEstatus", "text", sText);
  }
}

//------------------------------------------------------------------------------------------------------------------------

//@}
